<?php
namespace FiloBlu\BirthdayCoupon\Model\Magento\Quote\Quote;

use Magento\Quote\Model\Quote\Address\Total\Collector;
use Magento\Quote\Model\Quote\Address\Total\CollectorFactory;

class TotalsCollector extends \Magento\Quote\Model\Quote\TotalsCollector
{
    protected $_birthdayHelper;
    protected $totalCollector;
    protected $totalCollectorFactory;
    protected $eventManager;
    protected $storeManager;
    protected $totalFactory;
    protected $collectorList;
    protected $quoteValidator;
    protected $shippingFactory;
    protected $shippingAssignmentFactory;
    protected $_request;
    protected $_customerSession;

    public function __construct(
        \Magento\Customer\Model\Session $customerSession,
        \Magento\Framework\App\Request\Http $request,
        \FiloBlu\BirthdayCoupon\Helper\Data $birthdayHelper,
        Collector $totalCollector,
        CollectorFactory $totalCollectorFactory,
        \Magento\Framework\Event\ManagerInterface $eventManager,
        \Magento\Store\Model\StoreManagerInterface $storeManager,
        \Magento\Quote\Model\Quote\Address\TotalFactory $totalFactory,
        \Magento\Quote\Model\Quote\TotalsCollectorList $collectorList,
        \Magento\Quote\Model\ShippingFactory $shippingFactory,
        \Magento\Quote\Model\ShippingAssignmentFactory $shippingAssignmentFactory,
        \Magento\Quote\Model\QuoteValidator $quoteValidator
    ) {
        $this->_customerSession = $customerSession;
        $this->_request = $request;
        $this->_birthdayHelper = $birthdayHelper;
        $this->totalCollector = $totalCollector;
        $this->totalCollectorFactory = $totalCollectorFactory;
        $this->eventManager = $eventManager;
        $this->storeManager = $storeManager;
        $this->totalFactory = $totalFactory;
        $this->collectorList = $collectorList;
        $this->shippingFactory = $shippingFactory;
        $this->shippingAssignmentFactory = $shippingAssignmentFactory;
        $this->quoteValidator = $quoteValidator;
        parent::__construct($totalCollector, $totalCollectorFactory, $eventManager, $storeManager, $totalFactory, $collectorList, $shippingFactory, $shippingAssignmentFactory, $quoteValidator);
    }

    protected function _validateCouponCode(\Magento\Quote\Model\Quote $quote)
    {
        $code = $quote->getData('coupon_code');
        $addressHasCoupon = false;
        if (strlen($code)) {
            if($this->_birthdayHelper->isModuleEnabled() && $this->_birthdayHelper->isBirthdayCoupon($code)) {
                if(!$this->_customerSession->isLoggedIn() && $this->_request->getPostValue('customer_email')) {
                    $quote->setCustomerEmail($this->_request->getPostValue('customer_email'));
                }
                $validate = $this->_birthdayHelper->validateBirthdayCoupon($quote);
                if($validate->result == 'true') {
                    $addressHasCoupon = true;
                }
            }
            else {
                $addresses = $quote->getAllAddresses();
                if (count($addresses) > 0) {
                    foreach ($addresses as $address) {
                        if ($address->hasCouponCode()) {
                            $addressHasCoupon = true;
                        }
                    }
                }
            }

            if (!$addressHasCoupon) {
                $quote->setCouponCode('');
            }
        }
        return $this;
    }
}